@@ -2,7 +2,7 @@
 #'
 #' This is the main function for pedigree plotting, with many options for
 #' controlling the appearance of pedigree symbols and accompanying labels. Most
-#' of the work is done by the plotting functionality in the 'kinship2' package.
+#' of the work is done by the plotting functionality in the `kinship2` package.
 #'
 #' `plot.ped` is in essence an elaborate wrapper for
 #' [kinship2::plot.pedigree()].
@@ -11,13 +11,11 @@
 #' @param marker either a vector of names or indices referring to markers
 #'   attached to `x`, a `marker` object, or a list of such. The genotypes for
 #'   the chosen markers are written below each individual in the pedigree, in
-#'   the format determined by `sep` and `missing`. See also
-#'   `skipEmptyGenotypes` below. If NULL (the default), no genotypes are
-#'   plotted.
+#'   the format determined by `sep` and `missing`. See also `showEmpty`
+#'   below. If NULL (the default), no genotypes are plotted.
 #' @param sep a character of length 1 separating alleles for diploid markers.
 #' @param missing the symbol (integer or character) for missing alleles.
-#' @param skipEmptyGenotypes a logical. If TRUE, and `marker` is non-NULL,
-#'   empty genotypes (which by default looks like '-/-') are not printed.
+#' @param showEmpty a logical, indicating if empty genotypes should be included.
 #' @param labs a vector or function controlling the individual labels included
 #'   in the plot. Alternative forms:
 #'
@@ -40,11 +38,16 @@
 #'   members. For example if `col = list(red = "a", blue = c("b", "c"))` then
 #'   individual "a" will be red, "b" and "c" blue, and everyone else black. By
 #'   default everyone is drawn black.
-#' @param shaded a vector of ID labels indicating pedigree members whose plot
-#'   symbols should appear shaded.
+#' @param aff a vector of ID labels indicating pedigree members whose plot
+#'   symbols should be filled.
+#' @param hatched a vector of ID labels indicating pedigree members whose plot
+#'   symbols should be hatched.
+#' @param shaded (Deprecated) synonym of `hatched`
 #' @param deceased a vector of ID labels indicating deceased pedigree members.
 #' @param starred a vector of ID labels indicating pedigree members that should
 #'   be marked with a star in the pedigree plot.
+#' @param hints a list with alignment hints passed on to
+#'   `kinship2::align.pedigree()`. Usually not necessary.
 #' @param fouInb either "autosomal" (default), "x" or NULL. If "autosomal" or
 #'   "x", inbreeding coefficients are added to the plot above the inbred
 #'   founders. If NULL, or if no founders are inbred, nothing is added.
@@ -55,7 +58,8 @@
 #'   additional annotation.
 #' @param yadj A tiny adjustment sometimes needed to fix the appearance of
 #'   singletons.
-#' @param skip.empty.genotypes Deprecated; use `skipEmptyGenotype` instead.
+#' @param skipEmptyGenotypes Deprecated; use `showEmpty` instead.
+#' @param skip.empty.genotypes Deprecated; use `showEmpty` instead.
 #' @param id.labels Deprecated; use `labs` instead
 #' @param \dots arguments passed on to `plot.pedigree` in the `kinship2`
 #'   package. In particular `symbolsize` and `cex` can be useful.
@@ -74,9 +78,12 @@
 #' plot(x, marker = "SNP")
 #'
 #' # Other options
-#' plot(x, marker = "SNP", shaded = typedMembers(x),
+#' plot(x, marker = "SNP", hatched = typedMembers(x),
 #'      starred = "fa", deceased = "mo")
 #'
+#' # Filled symbols
+#' plot(x, aff = males(x))
+#'
 #' # Label only some members
 #' plot(x, labs = c("fa", "boy"))
 #'
@@ -87,7 +94,7 @@
 #' plot(x, labs = males)
 #'
 #' # Colours
-#' plot(x, col = list(red = "fa", green = "boy"), shaded = "boy")
+#' plot(x, col = list(red = "fa", green = "boy"), hatched = "boy")
 #'
 #' # Founder inbreeding is shown by default
 #' founderInbreeding(x, "mo") = 0.1
@@ -96,12 +103,30 @@
 #' # ... but can be suppressed
 #' plot(x, fouInb = NULL)
 #'
+#'
+#' #-----------------------------
+#' # In some cases, the plotting machinery of `kinship2` needs a hint
+#' # (see ?kinship2::align.pedigree)
+#'
+#' # Example with 3/4-siblings
+#' y = nuclearPed(2)
+#' y = addChildren(y, 3, mother = 5, nch = 1)
+#' y = addChildren(y, 4, mother = 5, nch = 1)
+#'
+#' plot(y) # bad
+#'
+#' hints = list(order = 1:7, spouse = rbind(c(3,5,0), c(5,4,0)))
+#' plot(y, hints = hints) # good
+#'
 #' @importFrom graphics text
 #' @export
-plot.ped = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyGenotypes = FALSE,
-                    labs = labels(x), title = NULL, col = 1, shaded = NULL, deceased = NULL,
-                    starred = NULL, fouInb = "autosomal", margins = c(0.6, 1, 4.1, 1),
-                    keep.par = FALSE, skip.empty.genotypes = NULL, id.labels = NULL, ...) {
+plot.ped = function(x, marker = NULL, sep = "/", missing = "-", showEmpty = FALSE,
+                    labs = labels(x), title = NULL, col = 1, aff = NULL, hatched = NULL,
+                    shaded = NULL, deceased = NULL,
+                    starred = NULL, hints = NULL, fouInb = "autosomal",
+                    margins = c(0.6, 1, 4.1, 1),
+                    keep.par = FALSE, skipEmptyGenotypes = NULL,
+                    skip.empty.genotypes = NULL, id.labels = NULL, ...) {
 
   if(!is.null(id.labels)) {
     message("The `id.labels` argument is deprecated in favor of `labs`, and will be removed in a future version")
@@ -111,8 +136,12 @@ plot.ped = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyGenotyp
   }
 
   if(!is.null(skip.empty.genotypes)) {
-    message("The `skip.empty.genotypes` argument has been renamed to `skipEmptyGenotypes`, and will be removed in a future version")
-    skipEmptyGenotypes = skip.empty.genotypes
+    message("The `skip.empty.genotypes` argument has been replaced by `showEmpty` and will be removed in a future version")
+    showEmpty = !skip.empty.genotypes
+  }
+  if(!is.null(skipEmptyGenotypes)) {
+    message("The `skipEmptyGenotypes` argument has been replaced by `showEmpty` and will be removed in a future version")
+    showEmpty = !skipEmptyGenotypes
   }
 
   if(hasSelfing(x))
@@ -167,7 +196,7 @@ plot.ped = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyGenotyp
 
     gg = do.call(cbind, lapply(mlist, format, sep = sep, missing = missing))
     geno = apply(gg, 1, paste, collapse = "\n")
-    if (skipEmptyGenotypes)
+    if (!showEmpty)
       geno[rowSums(do.call(cbind, mlist)) == 0] = ""
 
     text = if (!any(nzchar(text))) geno else paste(text, geno, sep = "\n")
@@ -191,18 +220,33 @@ plot.ped = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyGenotyp
      cols = rep(col, length = nInd)
   }
 
-  # Shading
-  if(is.function(shaded))
-    shaded = shaded(x)
-  if (!is.null(shaded)) {
+  # Very soft deprecation of `shaded`
+  if(!is.null(shaded)) {
+    hatched = shaded
+    shaded = NULL
+  }
+
+  # Affected/hatched individuals
+  if(is.function(aff))
+    aff = aff(x)
+  if(is.function(hatched))
+    hatched = hatched(x)
+  if(!is.null(aff) && !is.null(hatched))
+    stop2("Both `aff` and `hatched` cannot both be used")
+
+  if(!is.null(aff)) {
+    density = -1
+    angle = 90
+  }
+  else if(!is.null(hatched)) {
+    aff = hatched
     density = 25
     angle = 45
   } else {
-    density = NULL
-    angle = NULL
+    density = angle = NULL
   }
 
-  pedigree = as_kinship2_pedigree(x, deceased = deceased, shaded = shaded)
+  pedigree = as_kinship2_pedigree(x, deceased = deceased, aff = aff, hints = hints)
   pdat = kinship2::plot.pedigree(pedigree, id = text, col = cols, mar = margins,
                                  density = density, angle = angle, keep.par = keep.par, ...)
 
@@ -226,10 +270,11 @@ plot.ped = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyGenotyp
 
 #' @rdname plot.ped
 #' @export
-plot.singleton = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyGenotypes = FALSE,
-                          labs = labels(x), title = NULL, col = 1, shaded = NULL, deceased = NULL,
-                          starred = NULL, fouInb = "autosomal", margins = c(8, 0, 0, 0), yadj = 0,
-                          id.labels = NULL, ...) {
+plot.singleton = function(x, marker = NULL, sep = "/", missing = "-", showEmpty = FALSE,
+                          labs = labels(x), title = NULL, col = 1, aff = NULL,
+                          hatched = NULL, shaded = NULL,
+                          deceased = NULL, starred = NULL, fouInb = "autosomal",
+                          margins = c(8, 0, 0, 0), yadj = 0, id.labels = NULL, ...) {
 
   if(!is.null(id.labels)) {
     message("The `id.labels` argument is deprecated in favor of `labs`, and will be removed in a future version")
@@ -251,8 +296,16 @@ plot.singleton = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyG
   if(identical(labs, "num"))
     labs = c(`1` = labels(x))
 
-  if(is.function(shaded))
-    shaded = shaded(x)
+  if(is.function(aff))
+    aff = aff(x)
+
+  if(!is.null(shaded)) {
+    hatched = shaded
+    shaded = NULL
+    }
+
+  if(is.function(hatched))
+    hatched = hatched(x)
 
   if(is.function(starred))
     starred = starred(x)
@@ -279,8 +332,8 @@ plot.singleton = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyG
                                   verbose = FALSE))
 
   pdat = plot.ped(y, marker = y$MARKERS, sep = sep, missing = missing,
-               skipEmptyGenotypes = skipEmptyGenotypes, labs = labs,
-               title = title, col = col, shaded = shaded, deceased = deceased,
+               showEmpty = showEmpty, labs = labs,
+               title = title, col = col, aff = aff, hatched = hatched, shaded = shaded, deceased = deceased,
                starred = starred, margins = c(margins[1], 0, 0, 0), keep.par = TRUE, ...)
 
   usr = par("usr")
@@ -304,17 +357,22 @@ plot.singleton = function(x, marker = NULL, sep = "/", missing = "-", skipEmptyG
 
 #' @rdname plot.ped
 #' @export
-as_kinship2_pedigree = function(x, deceased = NULL, shaded = NULL) {
+as_kinship2_pedigree = function(x, deceased = NULL, aff = NULL, hints = NULL) {
     ped = as.data.frame(x)  # not as.matrix()
     ped$sex[ped$sex == 0] = 3 # kinship2 code for "diamond"
 
-    affected = ifelse(ped$id %in% shaded, 1, 0) # shaded=NULL => affected=c(0,0,..)
+    affected = ifelse(ped$id %in% aff, 1, 0) # NULL => affected01 = c(0,0,..)
     status = ifelse(ped$id %in% deceased, 1, 0)
 
-    suppressWarnings( # Avoid kinship2 warning about missing genders a.s.o.
+    kinped = suppressWarnings( # Avoid kinship2 warning about missing genders a.s.o.
       kinship2::pedigree(id = ped$id, dadid = ped$fid, momid = ped$mid,
                          sex = ped$sex, affected = affected,
                          status = status,  missid = 0))
+
+    # Possible hints for kinship2::align.pedigree
+    kinped$hints = hints
+
+    kinped
 }
 
 #' @rdname plot.ped
@@ -335,24 +393,29 @@ plot.pedList = function(x, ...) {
 #' Note that for tweaking dev.height and dev.width the function [dev.size()] is
 #' useful to determine the size of the active device.
 #'
-#' @param plot.arg.list A list of lists. Each element of `plot.arg.list` is a
-#'   list, where the first element is the [ped()] object to be plotted, and the
-#'   remaining elements are passed on to `plot.ped`. These elements must be
-#'   correctly named. See examples below.
+#' @param plots A list of lists. Each element of `plots` is a list, where the
+#'   first element is a pedigree, and the remaining elements are passed on to
+#'   `plot.ped`. These elements must be correctly named. See examples below.
 #' @param widths A numeric vector of relative widths of the subplots. Recycled
-#'   to `length(plot.arg.list)` if necessary, before passed on to [layout()].
-#'   Note that the vector does not need to sum to 1.
-#' @param frames Either a single logical (FALSE = no frames; TRUE = automatic
-#'   framing) or a list of numeric vectors: Each vector must consist of
-#'   consecutive integers, indicating subplots to be framed together. By default
-#'   the framing follows the list structure of `plot.arg.list`.
-#' @param frametitles A character vector of titles for each frame. If this is
-#'   non-NULL, titles for individuals subplots are ignored.
+#'   to `length(plots)` if necessary, before passed on to [layout()]. Note that
+#'   the vector does not need to sum to 1.
+#' @param groups A list of vectors, each consisting of consecutive integers,
+#'   indicating subplots to be grouped. By default the grouping follows the list
+#'   structure of `plots`.
+#' @param titles A character vector of titles for each group. Overrides titles
+#'   given in individuals subplots.
+#' @param frames A logical indicating if groups should be framed.
 #' @param fmar A single number in the interval \eqn{[0,0.5)} controlling the
 #'   position of the frames.
+#' @param frametitles Deprecated; use `titles` instead.
+#' @param source NULL (default), or the name or index of an element of `plots`.
+#'   If given, marker data is temporarily transferred from this to all the other
+#'   pedigrees. This may save some typing when plotting the same genotypes on
+#'   several pedigrees.
 #' @param newdev A logical, indicating if a new plot window should be opened.
 #' @param dev.height,dev.width The dimensions of the new plot window. If these
 #'   are NA suitable values are guessed from the pedigree sizes.
+#' @param verbose A logical.
 #' @param \dots Further arguments passed on to each call to [plot.ped()].
 #'
 #' @author Magnus Dehli Vigeland
@@ -360,39 +423,52 @@ plot.pedList = function(x, ...) {
 #' @seealso [plot.ped()]
 #'
 #' @examples
-#' # Simplest use: Just give a list of ped objects.
-#' # To guess suitable plot window dimensions, use 'newdev = TRUE'
+#' ##################
+#' # Basic examples #
+#' ##################
+#'
+#' # Simples use: Just give a list of ped objects.
 #' peds = list(nuclearPed(3), cousinPed(2), singleton(12), halfSibPed())
 #' plotPedList(peds, newdev = TRUE)
 #'
 #' # Modify the relative widths (which are not guessed)
-#' widths = c(2, 3, 1, 2)
-#' plotPedList(peds, widths = widths)
+#' w = c(2, 3, 1, 2)
+#' plotPedList(peds, widths = w)
 #'
 #' # In most cases the guessed dimensions are ok but not perfect.
 #' # Resize plot window manually and re-plot with `newdev = FALSE` (default)
-#' # plotPedList(peds, widths = widths)
+#' # plotPedList(peds, widths = w)
 #'
 #' ## Remove frames
-#' plotPedList(peds, widths = widths, frames = FALSE)
+#' plotPedList(peds, widths = w, frames = FALSE)
 #'
-#' # Non-default frames
-#' frames = list(1, 2:3)
-#' plotPedList(peds, widths = widths, frames = frames,
-#'             frametitles = c('First', 'Second'))
+#' # Non-default grouping
+#' plotPedList(peds, widths = w, groups = list(1, 2:3), titles = 1:2)
 #'
-#' # Parameters common to all plots can be added in the main call:
-#' margins = c(2, 4, 2, 4)
-#' title = 'Same title'
-#' labs = ''
-#' symbolsize = 1.5
-#' plotPedList(peds, widths = widths, frames = frames, margins = margins,
-#'             title = title, labs = labs, symbolsize = symbolsize,
-#'             newdev = TRUE)
+#' # Parameters added in the main call are used in each sub-plot
+#' plotPedList(peds, widths = w, margins = c(2, 4, 2, 4), labs = leaves,
+#'             hatched = leaves, symbolsize = 1.3, col = list(red = 1))
+#'
+#' dev.off()
+#'
+#' #################################
+#' # Example of automatic grouping #
+#' #################################
+#' H1 = nuclearPed()
+#' H2 = list(singleton(1), singleton(3))  # grouped!
+#'
+#' plotPedList(list(H1, H2), dev.height = 2, dev.width = 4,
+#'             titles = c(expression(H[1]), expression(H[2])))
+#'
+#' dev.off()
+#'
+#' ############################################################
+#' # Complex example with individual parameters for each plot #
+#' ############################################################
+#'
+#' # For more control of individual plots, each plot and all
+#' # its parameters can be specified in its own list.
 #'
-#' # COMPLEX EXAMPLE WITH MARKER DATA AND VARIOUS OPTIONS
-#' # For more control of individual plots, each plot and all its parameters
-#' # can be specified in its own list:
 #' x1 = nuclearPed(nch = 3)
 #' m1 = marker(x1, `3` = 1:2)
 #' marg1 = c(7, 4, 7, 4)
@@ -403,165 +479,208 @@ plot.pedList = function(x, ...) {
 #' m2 = marker(x2, alleles = "A")
 #' genotype(m2, leaves(x2)) = "A"
 #' marg2 = c(3, 4, 2, 4)
-#' plot2 = list(x2, marker = m2, margins = marg2, title = "Plot 2", symbolsize = 1.2,
-#'              skipEmptyGenotypes = TRUE, id = NULL)
+#' plot2 = list(x2, marker = m2, margins = marg2, title = "Plot 2",
+#'              symbolsize = 1.2, labs = NULL)
 #'
 #' x3 = singleton("Mr. X")
 #' marg3 = c(10, 0, 0, 0)
-#' plot3 = list(x3, margins = marg3, title = "Plot 3", symbolsize = 1, cex = 2)
+#' plot3 = list(x3, margins = marg3, title = "Plot 3",
+#'              symbolsize = 1, cex = 2)
 #'
 #' x4 = halfSibPed()
-#' shaded = 4:5
-#' col = c("black", "black", "black", "blue", "blue")
+#' hatched = 4:5
+#' col = list(red = founders(x4), blue = leaves(x4))
 #' marg4 = marg1
-#' plot4 = list(x4, margins = marg4, title = "Plot 4", shaded = shaded, col = col)
+#' plot4 = list(x4, margins = marg4, title = "Plot 4", cex = 1.3,
+#'              hatched = hatched, col = col)
 #'
 #' plotPedList(list(plot1, plot2, plot3, plot4), widths = c(2,3,1,2),
-#'             frames = list(1, 2:3, 4), newdev = TRUE)
+#'             groups = list(1, 2:3, 4), newdev = TRUE)
+#'
+#' dev.off()
+#'
+#' ################################
+#' # Example with large pedigrees #
+#' ################################
+#'
+#' # Important to set device dimensions here
 #'
-#' # Different example:
 #' plotPedList(list(halfCousinPed(4), cousinPed(7)),
-#'             title = c('Many generations', 'Very many generations'),
-#'             dev.height = 9, dev.width = 9)
+#'             titles = c("Large", "Very large"),
+#'             dev.height = 8, dev.width = 5)
 #'
+#' dev.off()
 #'
 #' @importFrom grDevices dev.new dev.size
 #' @importFrom graphics grconvertX grconvertY layout mtext rect par plot
 #' @export
-plotPedList = function(plot.arg.list, widths = NA, frames = TRUE,
-                       frametitles = names(plot.arg.list), fmar = NA,
-                       dev.height = NA, dev.width = NA,
-                       newdev = !is.na(dev.height) || !is.na(dev.width),
-                       ...) {
+plotPedList = function(plots, widths = NULL, groups = NULL, titles = NULL,
+                       frames = TRUE, fmar = NULL, frametitles = NULL,
+                       source = NULL, dev.height = NULL, dev.width = NULL,
+                       newdev = !is.null(dev.height) || !is.null(dev.width),
+                       verbose = FALSE, ...) {
 
-  plotlist.flattened = list()
-  if (deduceFrames <- isTRUE(frames)) {
-    frames = list()
+  if(!is.null(frametitles)) {
+    message("Argument `frametitles` is deprecated; use `titles` instead")
+    titles = frametitles
+  }
+
+  if(!(isTRUE(frames) || isFALSE(frames))) {
+    message("`frames` must be either TRUE or FALSE; use `groups` to specify framing groups")
+    groups = frames
+    frames = TRUE
+  }
+
+  # If explicit source given, transfer marker data to all
+  if(!is.null(source)) {
+    srcPed = plots[[source]]
+    if(is.null(srcPed))
+      stop2("Unknown source pedigree: ", source)
+    if(nMarkers(srcPed) == 0)
+      stop2("The source pedigree has no attached markers")
+    plots = lapply(plots, transferMarkers, from = srcPed)
+  }
+
+  deduceGroups = is.null(groups)
+  if (deduceGroups) {
+    groups = list()
     k = 0
   }
-  for (p in plot.arg.list) {
+
+  # Flatten plot list
+  flatlist = list()
+
+  for (p in plots) {
     if (is.ped(p))
-      p = list(p)  # will now be included in next line
-    if (is.pedList(p)) {
-      plotlist.flattened = c(plotlist.flattened, lapply(p, list))
+      newpeds = list(list(p))
+    else if (is.pedList(p))
+      newpeds = lapply(p, list)
+    else { # if list of ped with plot arguments
+      if (!is.ped(p[[1]]))
+        stop2("First element must be a `ped` object", p[[1]])
+      newpeds = list(p)
     }
-    else {
-        # if list of ped with plot arguments
-        if (!is.ped(p[[1]]))
-          stop2("First element must be a `ped` object", p[[1]])
-        p = list(p)
-        plotlist.flattened = append(plotlist.flattened, p)
-      }
-    if (deduceFrames) {
-      group = (k + 1):(k <- k + length(p))
-      frames = append(frames, list(group))
+
+    flatlist = c(flatlist, newpeds)
+    if (deduceGroups) {
+      groups = c(groups, list(k + seq_along(newpeds)))
+      k = k + length(newpeds)
     }
   }
 
-  N = length(plotlist.flattened)
-  if (identical(widths, NA))
-    widths = vapply(plotlist.flattened, function(p) ifelse(is.singleton(p[[1]]), 1, 2.5), 1)
-  else
+  N = length(flatlist)
+  NG = length(groups)
+
+  # Groups: check that each vector is consecutive integers, and no duplicates.
+  for (v in groups)
+    if (!is.numeric(v) || !isTRUE(all.equal.numeric(v, v[1]:v[length(v)])))
+      stop2("Each element of `groups` must consist of consecutive integers: ", v)
+  dup = anyDuplicated.default(unlist(groups))
+  if (dup > 0)
+    stop2("Plot occurring twice in `groups`: ", dup)
+
+  # Group titles
+  grouptitles = titles %||% names(plots)
+  if (!is.null(grouptitles) && length(grouptitles) != NG)
+    stop2(sprintf("Length of `titles` (%d) does not equal number of groups (%d)",
+                  length(grouptitles), NG))
+
+  # If no group titles, use inner titles if present
+  finalTitles = grouptitles %||% sapply(flatlist, function(p) p$title %||% "")
+  hasTitles = any(nchar(finalTitles) > 0)
+
+  # Relative plot widths
+  if (is.null(widths))
+    widths = vapply(flatlist, function(p) ifelse(is.singleton(p[[1]]), 1, 2.5), 1)
+  else {
+    if(!is.numeric(widths) && !length(widths) %in% c(1,N))
+      stop2("`widths` must be a numeric of length either 1 or the total number of objects")
     widths = rep_len(widths, N)
-  maxGen = max(vapply(plotlist.flattened, function(arglist) .generations(arglist[[1]]), 1))
+  }
+
+  maxGen = max(vapply(flatlist, function(arglist) .generations(arglist[[1]]), 1))
 
-  if (hasframetitles <- !is.null(frametitles))
-    if(length(frametitles) != length(frames))
-      stop2(sprintf("Length of `frametitles` (%d) does not equal number of frames (%d)",
-            length(frametitles), length(frames)))
 
   extra.args = list(...)
-  if (!"title" %in% names(extra.args))
-    extra.args$title = ""
 
-  defaultmargins = if (N > 2)
-    c(0, 4, 0, 4) else c(0, 2, 0, 2)
+  defaultmargins = if (N > 2) c(0, 4, 0, 4) else c(0, 2, 0, 2)
 
-  plotlist.flattened = lapply(plotlist.flattened, function(arglist) {
+  # Prepare plot args for each ped
+  plotlist = lapply(flatlist, function(arglist) {
     names(arglist)[1] = "x"
-    g = .generations(arglist$x)
-    addMargin = 2 * (maxGen - g + 1)
-    if (!"margins" %in% c(names(arglist), names(extra.args)))
-      arglist$margins = defaultmargins + c(addMargin, 0, addMargin, 0)
 
-    # additional arguments given in (...)
+    # Additional arguments given in (...)
     for (parname in setdiff(names(extra.args), names(arglist)))
       arglist[[parname]] = extra.args[[parname]]
+
+    # Remove inner title
+    arglist$title = NULL
+
+    # Margins
+    arglist$margins = arglist$margins %||% {
+      g = .generations(arglist$x)
+      addMar = 2 * (maxGen - g + 1)
+      defaultmargins + c(addMar, 0, addMar, 0)
+    }
+
     arglist
   })
 
-  # title: this must be treated specially (in outer margins)
-  titles = sapply(plotlist.flattened, "[[", "title")
-  plotlist.flattened = lapply(plotlist.flattened, function(arglist) {
-    arglist$title = ""
-    arglist
-  })
-  hastitles = hasframetitles || any(titles != "")
-
-  # frame list: check that each vector is consecutive integers, and no duplicates.
-  if (is.list(frames)) {
-    for (v in frames)
-      if (!identical(TRUE, all.equal(v, v[1]:v[length(v)])))
-        stop2("Each element of `frames` must consist of consecutive integers: ", v)
-    dup = anyDuplicated.default(unlist(frames))
-    if (dup > 0)
-      stop2("Plot occurring twice in `frames` list: ", dup)
-  }
-
-
-  # create layout of plot regions and plot!
+  # Layout of plot regions
   if (newdev) {
-    if (is.na(dev.height))
-      dev.height = max(3, 1 * maxGen) + 0.3 * as.numeric(hastitles)
-    if (is.na(dev.width))
-      dev.width = 3 * N
+    dev.height = dev.height %||% {max(3, 1 * maxGen) + 0.3 * as.numeric(hasTitles)}
+    dev.width = dev.width %||% {3 * N}
     dev.new(height = dev.height, width = dev.width, noRStudioGD = TRUE)
   }
 
-  new.oma = if (hastitles) c(0, 0, 3, 0) else c(0, 0, 0, 0)
+  new.oma = if (hasTitles) c(0, 0, 3, 0) else c(0, 0, 0, 0)
   opar = par(oma = new.oma, xpd = NA)
   on.exit(par(opar))
 
+  if(verbose) {
+    message("Group structure: ", toString(groups))
+    message("Relative widths: ", toString(widths))
+    message("Default margins: ", toString(defaultmargins))
+    message("Indiv. margins:")
+    for(p in plotlist) message("  ", toString(p$margins))
+    message("Input width/height: ", toString(c(dev.width, dev.height)))
+    message("Actual dimensions: ", toString(round(dev.size(),3)))
+  }
+
+  # Plot!
   layout(rbind(1:N), widths = widths)
-  for (arglist in plotlist.flattened)
+  for (arglist in plotlist)
     do.call(plot, arglist)
 
-  # leftmost coordinate of each plot region (converted to value in [0,1]).
+  # Leftmost coordinate of each plot region (converted to value in [0,1]).
   ratios = c(0, cumsum(widths)/sum(widths))
 
-  # add frames
-  if (is.list(frames)) {
-    midpoints = numeric()
-    fstart_index = sapply(frames, function(v) v[1])
-    fstop_index = sapply(frames, function(v) v[length(v)])
-    ratio_start = ratios[fstart_index]
-    ratio_stop = ratios[fstop_index + 1]  # fordi 0 foerst
-    midpoints = (ratio_start + ratio_stop)/2
+  # Group coordinates
+  grStartIdx = sapply(groups, function(v) v[1])
+  grStopIdx = sapply(groups, function(v) v[length(v)])
+  grStart = ratios[grStartIdx]
+  grStop = ratios[grStopIdx + 1]  # since 0 first
+
+  # Draw frames
+  if(frames) {
+    # Default margin: 5% of vertical height, but at most 0.25 inches.
+    fmar = fmar %||% min(0.05, 0.25/dev.size()[2])
 
-    # margin (fmar): if NA, set to 5% of vertical height, but at most 0.25 inches.
-    if (is.na(fmar))
-      fmar = min(0.05, 0.25/dev.size()[2])
     margPix = grconvertY(0, from = "ndc", to = "device") * fmar
     margXnorm = grconvertX(margPix, from = "device", to = "ndc")
-    frame_start = grconvertX(ratio_start + margXnorm, from = "ndc")
-    frame_stop = grconvertX(ratio_stop - margXnorm, from = "ndc")
+    frame_start = grconvertX(grStart + margXnorm, from = "ndc")
+    frame_stop = grconvertX(grStop - margXnorm, from = "ndc")
     rect(xleft = frame_start,
          ybottom = grconvertY(1 - fmar, from = "ndc"),
          xright = frame_stop,
          ytop = grconvertY(fmar, from = "ndc"), xpd = NA)
   }
 
-  cex.title =
-    if ("cex.main" %in% names(extra.args)) extra.args$cex.main
-    else NA
-
-  if (hasframetitles) {
-    for (i in 1:length(frames))
-      mtext(frametitles[i], outer = TRUE, at = midpoints[i], cex = cex.title)
-  }
-  else if (hastitles) {
-    for (i in 1:N)
-      mtext(titles[i], outer = TRUE, at = (ratios[i] + ratios[i + 1])/2, cex = cex.title)
+  # Add titles
+  if(hasTitles) {
+    midpoints = if(!is.null(grouptitles)) (grStart + grStop)/2 else ratios[1:N] + diff(ratios)/2
+    cex.title = extra.args$cex.main %||% NA
+    mtext(finalTitles, outer = TRUE, at = midpoints, cex = cex.title)
   }
 }
 
