@@ -19,15 +19,15 @@ stopifnot2 = function(...) {
   }
 }
 
-# Test that input is a single positive (or similar) integer.
-is_count = function(x, minimum = 1) {
+# Test that input is a single integer.
+isCount = function(x, minimum = 1, maximum = NA) {
   isTRUE(length(x) == 1 &&
          (is.integer(x) || (is.numeric(x) && x == as.integer(x))) &&
-         x >= minimum)
+         x >= minimum && (is.na(maximum) || x <= maximum))
 }
 
 # Test that input is a single number, with optional range constraints
-is_number = function(x, minimum = NA, maximum = NA) {
+isNumber = function(x, minimum = NA, maximum = NA) {
   isTRUE(length(x) == 1 &&
          is.numeric(x) &&
          (is.na(minimum) || x >= minimum) &&
@@ -38,6 +38,12 @@ is_number = function(x, minimum = NA, maximum = NA) {
   if(is.null(x)) y else x
 }
 
+seq_ped = function(x)
+  seq_len(pedsize(x))
+
+seq_markers = function(x)
+  seq_len(nMarkers(x))
+
 # A safer version of base::sample
 safe_sample <- function(x, ...) x[sample.int(length(x), ...)]
 
@@ -120,7 +126,10 @@ commentAndRealign = function(df, i, pred, comment) {
     return(df)
 
   df[[i]] = paste0(df[[i]], ifelse(pred, comment, padding))
-  names(df)[i] = paste0(names(df)[i], padding)
+
+  if(!is.null(names(df)))
+    names(df)[i] = paste0(names(df)[i], padding)
+
   df
 }
 
