@@ -3,8 +3,15 @@ context("singletons")
 s = singleton(1)
 
 test_that("singletons are constructed", {
-  expect_error(s, NA)
-  expect_error(singleton("A", sex=2, famid="FAM"), NA)
+  expect_s3_class(s, "singleton")
+  expect_identical(singleton("A", sex=2, famid="FAM"), ped("A", 0,0,2,famid= "FAM"))
+})
+
+test_that("singleton() catches bad input", {
+  expect_error(singleton(1:2), "`id` must have length 1")
+  expect_error(singleton(1, sex = 1:2), "`sex` is longer than the number of individuals")
+  expect_error(singleton(1, sex = 3), "Illegal sex")
+  expect_error(singleton(1, sex = "male"), "Illegal sex")
 })
 
 test_that("singleton modifications commute", {
